//******************************************************************************
// Test Program for the C Function Library in Spin
// Author: Dave Hein
// Copyright (c) 2010
// See end of file for terms of use.
//******************************************************************************
//******************************************************************************
// Revison History
// v1.0 - 4/2/2010 First official release
//******************************************************************************
/*
  This is a test program for the C function library written in spin.  This
  program tests out the various string, I/O and memory allocation functions
  that are in the library.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>

int waitflag = 0;
int stack1[100];
int stack2[100];
int stack3[100];
int stack4[100];

#ifndef CSPIN_FLAG
int cogid;
int cnt;
int clkfreq;
#endif

void MultiCogPrintTest(void);
int cognew(void *, void *);
FILE *c_openserial(int, int, int, int, int, int);
int isdigit(int);
void waitcnt(int);

void main(void)
{
    int i, j;
    char *str;
    char *str1;
    char *arglist[10];
    int x;
    FILE *port;

// Wait for a key press
    getchar();

// Allocate 100 bytes of memory and test strcpy, strcat and printf
    str = malloc(100);
    strcpy(str, "This");
    strcat(str, " is");
    strcat(str, " a");
    strcat(str, " test.\n");
    printf(str);

// Use gets and puts to read a string and print it out
    puts("Enter string: ");
    gets(str);
    puts(str);
    putchar(13);

// Read two numbers and print them out using scanf and printf
    puts("Enter a decimal number and a hex number: ");
    scanf("%d %x", &i, &j);
    printf("%d %x\n", i, j);

// Read a floating point number using gets and sscanf, and print it out with printf
    puts("Enter a floating point number: ");
    gets(str);
    sscanf(str, "%f", &x);
    printf("%f %e\n", x, x);

// Print out floating point numbers from 2^-32 to 2^32
    printf("Press any key to run the floating-point print test\n");
    getchar();
    for (i = -32; i <= 32; i++)
    {
        x = 1.0;
        x += i << 23; // Adjust the exponent
        printf("2.0^(%3d) = %16.5f, %13.6e\n", i, x, x);
    }

// Test itoa by printing out 0x6789abcd in base 16, 8, 4 and 2
    printf("Press any key to continue\n");
    getchar();
    j = 0x6789abcd;
    for (i = 16; i > 1; i >>= 1)
    {
        itoa(j, str, i);
        printf("%s, base %d, len = %d\n", str, i, strlen(str));
    }

// Allocate 2000 bytes for str1, and test strcmp and strncmp
    str1 = malloc(2000);
    strcpy(str, "+12345");
    strcpy(str1, "+1234");
    for (i = 0; i < 3; i++)
    {
        printf("strlen(%s) = %d\n", str1, strlen(str1));
        printf("strcmp(%s, %s) = %d\n", str, str1, strcmp(str, str1));
        printf("strncmp(%s, %s, 6) = %d\n", str, str1, strncmp(str, str1, 6));
        strcat(str1, "5");
    }

// Test isdigit function
    printf("isdigit(%c) = %d\n", str[0], isdigit(str[0]));
    printf("isdigit(%c) = %d\n", str[1], isdigit(str[1]));

// Test sprintf
    sprintf(str, "sprintf test %03d %.3d\n", 34, 35);
    puts(str);

// Test vprintf
    for (i = 0; i < 10; i++)
    {
        arglist[i] = (char *)i;
    }
    vprintf("%d %d %d %d %d %d %d %d %d %d\n", arglist);

// Test malloc and free
    printf("Press any key to run the malloc test\n");
    getchar();
    printf("malloc(1000) = %04x\n", arglist[0] = malloc(1000));
    printf("malloc(2000) = %04x\n", arglist[1] = malloc(2000));
    printf("malloc(4000) = %04x\n", arglist[2] = malloc(4000));
    printf("malloc(8000) = %04x\n", arglist[3] = malloc(8000));
    printf("malloc(16000) = %04x\n", arglist[4] = malloc(16000));
    printf("free(%04x)\n", arglist[3]);
    free(arglist[3]);
    printf("free(%04x)\n", arglist[2]);
    free(arglist[2]);
    printf("malloc(16000) = %04x\n", arglist[4] = malloc(16000));
    for (i = 0; i < 5; i++)
    {
        free(arglist[i]);
    }

// Open a new serial port and test it
    printf("Second Seial Port Test\n");
    printf("Press any key when ready\n");
    getchar();
    // **** Change from c_openserial to c.openserial in the Spin file
    port = c_openserial(16, 16, 0, 300, 128, 128);
    for (i = 1; i <= 10; i++)
    {
        vfprintf(port, "Test line %d\n", (char **)(&i));
        fgets(str, 100, port);
        printf("received: %s\n", str);
    }

// Test multi-cog prints
    printf("Press any key to start the multi-cog print test\n");
    getchar();
    waitflag = 1;
    cognew(MultiCogPrintTest, stack1);
    cognew(MultiCogPrintTest, stack2);
    cognew(MultiCogPrintTest, stack3);
    cognew(MultiCogPrintTest, stack4);
    MultiCogPrintTest();

    printf("Test program completed\n");
}

// This routine waits for cog 0 to clear the waitflag, and all the cogs print at the same time
void MultiCogPrintTest(void)
{
    int i;

    for (i = 1; i <= 4; i++)
    {
        if (cogid == 0)
          waitflag = 0;
        while (waitflag);
        printf("Print %d from cog %d\n", i, cogid);
        waitflag = 1;
        if (cogid == 0)
	{
            waitcnt(clkfreq+cnt);
            putchar(13);
	}
    }
}
